/***
* Menu1.prg
* NOTES......: Compile with /a /m /n /w /dMENU_TEST
* Pull-down menu code with left and right arrow
* switching between pull-downs
*/

#ifdef MENU_TEST

    #include "Box.ch"
    #define BACKGROUND 176

    FUNCTION MenuTest

    LOCAL aVert1, aVert2, aVert3, aVert4, aHoriz

      SET CURSOR OFF
      SetColor("W+/B,W+/RB,B,B,GR+/B")
      @ 0, 0, 24, 79 BOX replicate(chr(BACKGROUND), 9)

      // ***** To load and run prompts from a database, use:
      DoMenu(BuildMenu("MAIN_MENU"))
      // ***************************************************
      WAIT

      // Here we hard code the prompts in an array
      aVert1 := { ;
                  {"By Date",  {|| PayByDate()  } }, ;
                  {"By Name",  {|| PayByName()  } }, ;
                  {"By Value", {|| PayByValue() } }  ;
                }

      aVert2 := { ;
                  {"By Date",   {|| RecByDate()  } }, ;
                  {"By Name",   {|| RecByName()  } }, ;
                  {"By Value",  {|| RecByValue() } }  ;
                }

      aVert4 := { ;
                  {"By Name", {|| CustByName() }  }, ;
                  {"By City", {|| CustByCity() }  }  ;
                }

// Extended definition
      aVert3 := { ;
                  { "Quick Summary",        {|| RecByDate() } }, ;
                  { "Detailed Payables",    {|| DetPay()    } }, ;
                  { "Detailed Receivables", {|| DetRec()    } }, ;
                  { "Customer List",        aVert4            }  ;
                }
/*
      aVert3 := { ;
                  { "Quick Summary",        {|| RecByDate() } }, ;
                  { "Detailed Payables",    {|| DetPay()    } }, ;
                  { "Detailed Receivables", {|| DetRec()    } }, ;
                  { "Customer List",        {|| CustList()  } }  ;
                }
*/

      aHoriz := { ;
                  {"Customers",   aVert1}, ;
                  {"Receivables", aVert2}, ;
                  {"Payables",    aVert3} ;
                }

      SET WRAP ON

      DoMenu(aHoriz)

    RETURN NIL
#endif

#include "Commands.ch"
#include "Inkey.ch"

/***
* VOID do_menu(prompts)
*
* ARRAY prompts      - An array structured as a menu
*
*
* Given an array in the form of a 2 - D menu, issue the top
* level prompts, then either execute the lower level menu or
* invoke the code block.
*/

FUNCTION DoMenu(aPrompts)

LOCAL nPrompts, nPrompt, cSaveWin, nChoice, aCols

    nPrompts := Len(aPrompts)

    // We need to tell any lower level prompt which
    // Column to start in. They should start in the same
    // column as the top level prompt. We thus need to keep track
    // of the start columns of each horizontal prompt. The array
    // aCols does this.

    aCols := Array(nPrompts)

    // The top level menu is the width of the screen and is framed.
    // The prompts appear on line 2. Save the area we will overwrite.
    cSaveWin := SaveScreen(0, 0, 3, MaxCol())

    // First frame and clear the area ...
    @ 0, 0 CLEAR TO 2, MaxCol()
    @ 0, 0 TO 2, MaxCol()

    // Start on the first prompt
    nChoice := 1

    * Until the user escapes ...
    DO WHILE nChoice != 0
        // Set the cursor position for the menu-drawing
        @ 1, 0 SAY ""

        // Issue each prompt on row 1, separated by two spaces.
        // Save the start column in the cols array
        FOR nPrompt := 1 TO nPrompts
            aCols[nPrompt] := Col() + 2
            @ 1, col() + 2 PROMPT aPrompts[nPrompt, 1]
        NEXT

        // Invoke the menu, starting with the current prompt
        // Initially this is 1
        MENU TO nChoice

        // If the user did not escape from the menu ...
        IF nChoice > 0
            // If the corresponding entry is a block, evaluate
            // it (calling its function)
            IF ValType(aPrompts[nChoice, 2]) = "B"
                Eval(aPrompts[nChoice, 2])
            ELSE
                // Otherwise call the lower level pull - menu routine
                // We have to pass it the array of prompts / blocks,
                // and the menu's start row and column. This
                // is the top-left location of the pull-downs
                // box frame.
                doPullDown(aPrompts[nChoice, 2], 3, aCols[nChoice])
            ENDIF
        ENDIF
    ENDDO

    // Restore the area overwritten by the horizontal menu
    RestScreen(0, 0, 3, MaxCol(), cSaveWin)

RETURN NIL


// Invoke pull down menu, structure defined by mprompts, at
// row t, column l.

FUNCTION DoPullDown(aPrompts, t, l)

STATIC nDepth := 0

LOCAL cSaveWin, nLenLongest, nPrompts, b, r, nPrompt, nChoice
LOCAL bSaveleft, bSaveRight

    nDepth++

    // The width of the window is determined by the longest prompt
    nLenLongest := LenLPrompt(aPrompts)

    // The height of the window is determined by the number of prompts
    nPrompts := Len(aPrompts)

    // Set the bottom row number for the frame ...
    b := t + nPrompts + 1

    // Set the ending column number the frame ...
    r := l + nLenLongest + 1

    // First save the area we will overwrite
    cSaveWin := SaveScreen(t, l, b, r)

    // Now frame it
    @ t, l TO b, r

    // Start at the first prompt
    nChoice = 1

    // Until the user escapes ...
    DO WHILE nChoice != 0
        // Issue each prompt, padding with spaces to the width
        // of the longest.
        FOR nPrompt := 1 TO nPrompts
            @ t + nPrompt, l + 1 PROMPT ;
                 Pad(aPrompts[nPrompt, 1], nLenLongest)
        NEXT

        // Invoke the menu, starting with the current prompt
        // Initially this is 1
        IF nDepth == 1
            bSaveleft  := SetKey(K_LEFT,   {|| leftstuff()})
            bSaveright := SetKey(K_RIGHT,  {|| rightstuff()})
        ELSE
            bSaveleft  := SetKey(K_LEFT,   {|| NIL })
            bSaveright := SetKey(K_RIGHT,  {|| NIL })
        ENDIF
        MENU TO nChoice

        SetKey(K_LEFT,  bSaveLeft)
        SetKey(K_RIGHT, bSaveRight)

        // If the user did not escape from the menu ...
        IF nChoice > 0
            // If the corresponding entry is a block, evaluate
            // it (calling its function)
            IF ValType(aPrompts[nChoice, 2]) = "B"
                Eval(aPrompts[nChoice, 2])
            ELSE
                // Otherwise call the lower level pull - menu routine
                // We have to pass it the array of prompts / blocks,
                // and the menu's start row and column. This
                // is the top-left location of the pull-downs
                // box frame. We start at 2 / 3 s across the selected
                // parent prompt, on th eline below
                DoPullDown(aPrompts[nChoice, 2], ;
                             nChoice + t + 1, ;
                             l + 1 + 2 * (nLenLongest / 3))
            ENDIF
        ENDIF
    ENDDO

    * Now restore the window we saved
    RestScreen(t, l, b, r, cSaveWin)

    nDepth--

RETURN NIL


STATIC FUNCTION LeftStuff

    KEYBOARD chr(K_ESC) + chr(K_LEFT) + chr(K_ENTER)

RETURN NIL


STATIC FUNCTION RightStuff

    KEYBOARD chr(K_ESC) + chr(K_RIGHT) + chr(K_ENTER)

RETURN NIL


STATIC FUNCTION LenLPrompt(aPrompts)

LOCAL cLongest, i

    cLongest := aPrompts[1, 1]
    FOR i := 1 TO Len(aPrompts)
        IF Len(cLongest) < len(aPrompts[i, 1])
            cLongest := aPrompts[i, 1]
        ENDIF
    NEXT

RETURN len(cLongest)


// Following routine allows menu to be located in a database.
// It reads the database and builds the array required by
// do_menu.


// Index key value for the menu database index
#define IKEY_VAL(m_name) pad(upper(m_name), Len(menu_name))

FUNCTION BuildMenu(cMenuName)

FIELD menu_name, prompt, next_menu, block IN menu

LOCAL nSaveSel, aMenu := {}

    nSaveSel := Select()
    USE Menu
    IF !File("Menu.ntx")
      INDEX ON Upper(menu_name) TO Menu
    ELSE
      SET INDEX TO Menu
    ENDIF
    aMenu := GetPrompts(cMenuName)
    CLOSE menu

    SELECT (nSaveSel)

RETURN aMenu


STATIC FUNCTION GetPrompts(cMenuName)

FIELD menu_name, prompt, next_menu, block IN menu

LOCAL aPrompts := {}, ;
      nSaveRecno := recno()

    SEEK IKEY_VAL(cMenuName)

    DO WHILE IKEY_VAL(menu_name) == IKEY_VAL(cMenuName)
        Aadd(aPrompts, {trim(prompt), Iif(Empty(next_menu), ;
                                       COMPILE(block), ;
                                       GetPrompts(next_menu)) ;
                        })
        SKIP
    ENDDO

    GOTO nSaveRecno

RETURN aPrompts

// Alternative version using DBEval()
/*
STATIC FUNCTION GetPrompts(cMenuName)

FIELD menu_name, prompt, next_menu, block IN menu

LOCAL aPrompts := {}, ;
      nSaveRecno := recno(), bWhileScope

    SEEK IKEY_VAL(m_name)
    cMenuName := {||IKEY_VAL(menu_name) == IKEY_VAL(cMenuName)}
    DbEval({|| Aadd(aPrompts, {Trim(prompt), ;
                                Iif(Empty(next_menu), ;
                                    COMPILE(block), ;
                                    GetPrompts(next_menu))}) ;
                               },, bWhileScope,,, .F.)

    GOTO nSaveRecno

RETURN aPrompts

*/

FUNCTION pay_30comp

    msg(procname())

RETURN NIL

FUNCTION pay_30date

    msg(procname())

RETURN NIL

FUNCTION cust_byname

    msg(procname())

RETURN NIL

FUNCTION cust_byid

    msg(procname())

RETURN NIL


FUNCTION cust_bycity

    msg(procname())

RETURN NIL

FUNCTION rec_byinv

    msg(procname())

RETURN NIL


FUNCTION rec_bycon

    msg(procname())

RETURN NIL

FUNCTION rec_cur

    msg(procname())

RETURN NIL


FUNCTION rec_30

    msg(procname())

RETURN NIL


FUNCTION rec_60

    msg(procname())

RETURN NIL


FUNCTION pay_comp

    msg(procname())

RETURN NIL


FUNCTION pay_cont

    msg(procname())

RETURN NIL

FUNCTION pay_cur

    msg(procname())

RETURN NIL


FUNCTION pay_60

    msg(procname())

RETURN NIL

FUNCTION msg(cFuncName)

    SAVE SCREEN
    @ 10, 10 SAY cFuncName
    inkey(0)
    RESTORE SCREEN

RETURN NIL

PROC PayByDate
  Msg(ProcName())
RETURN

PROC PayByName
  Msg(ProcName())
RETURN

PROC PayByValue
  Msg(ProcName())
RETURN

PROC RecByName()
  Msg(ProcName())
RETURN

PROC RecByValue
  Msg(ProcName())
RETURN

PROC RecByDate
  Msg(ProcName())
RETURN

PROC DetPay
  Msg(ProcName())
RETURN

PROC DetRec
  Msg(ProcName())
RETURN

PROC CustList()
  Msg(ProcName())
RETURN

PROC CustByName()
  Msg(ProcName())
RETURN

PROC CustByCity()
  Msg(ProcName())
RETURN
